-------------------------------------------------------------------------------
--|> Events
-- Manages all events
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local UnitIsPVPSanctuary = UnitIsPVPSanctuary
local UnitIsEnemy = UnitIsEnemy
local GetTime = GetTime
local UnitIsPlayer = UnitIsPlayer
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitHealthMax = UnitHealthMax
local UnitLevel = UnitLevel
local IsInInstance = IsInInstance
local bit_band = bit.band
local C_Timer_After = C_Timer.After
-- local InCombatLockdown = InCombatLockdown

-------------------------------------------------------------------------------
--|> LIBS
-------------------------------------------------------------------------------
local LibCLHealth = LibStub("LibCombatLogHealth-1.0")

-------------------------------------------------------------------------------
--|> FUNCTIONS
-------------------------------------------------------------------------------

--=> Static Role Assignment
---------------------------------------------------------------------
function weizPVP:RoleAssign(class)
    local role = nil
    if class == "ROGUE" or class == "MAGE" or class == "WARLOCK" or class == "HUNTER" then
        role = "DAMAGER"
    end
    return role
end

--=> Get Spec from Spell
-----------------------------------------------------------------------
function weizPVP:GetSpecFromSpell(SpellID)
    local specializationID = weizPVP.SpecDB[SpellID]
    return specializationID
end
--=> Get Role from Spec
-----------------------------------------------------------------------
function weizPVP:GetSpecRole(specializationID)
    local _, _, _, _, role, _, _ = GetSpecializationInfoByID(specializationID)
    return role
end

--=> Get PVP Zone
-----------------------------------------------------------------------
local function GetPVPZone()
    --> Get for instance
    local inInstance = select(1, IsInInstance()) -- nil if NOT in an instance
    local inWorld = false
    if inInstance == false then
        inWorld = true
    end
    local _, instanceType = GetInstanceInfo()

    --> Check if WarMode
    local PlayerWarModeEnabled = UnitIsWarModeDesired("player")
    local InWorldPVPZone = inWorld and PlayerWarModeEnabled

    if InWorldPVPZone == true or weizPVP.Options.GLOBAL_DEBUG == true or (instanceType == "pvp" and weizPVP.Options.EnableBGs == true) then
        if weizPVP.Options.Enabled == false then
            weizPVP:OnEnable()
        end
    else
        if weizPVP.Options.Enabled == true then
            weizPVP:OnDisable()
        end
    end
end

-------------------------------------------------------------------------------
--|> EVENTS
-------------------------------------------------------------------------------

--=> EVENT: COMBAT_LOG_UNFILTERED
-----------------------------------------------------------------------
function weizPVP:CombatLogEvent()
    local _, --timestamp
        event,
        _, --hideCaster
        srcGUID,
        srcName,
        srcFlags,
        _, --sourceRaidFlags
        dstGUID,
        dstName,
        dstFlags,
        _, --destRaidFlags
        arg12, --spellID
        arg13, --spellName
        _,
        _,
        _ = CombatLogGetCurrentEventInfo()

    --> Dont scan yourself when you are both the source and destination
    if (dstName == srcName and srcName == weizPVP.Player.Name) and weizPVP.Options.GLOBAL_DEBUG == false then
        return
    end

    local OBJECT_TYPE_PLAYER = 0x00000400
    local OBJECT_TYPE_ENEMY = 0x00000040

    --> Source Unit Info
    -------------------------------------------------------------------------------------------------------------------
    if (bit_band(srcFlags, OBJECT_TYPE_ENEMY) ~= 0 or weizPVP.Options.GLOBAL_DEBUG) and bit_band(srcFlags, OBJECT_TYPE_PLAYER) ~= 0 and srcGUID and srcName and srcName ~= "Unknown" then
        local newDataFound = false
        local role = nil
        local stealth = nil
        local dead = false

        --> If Player doesn't exist in DB
        if not weizPVP.PlayerDB[srcName] then
            newDataFound, role = weizPVP:GetLevelData(true, event, srcName, srcGUID, arg12, arg13)
        elseif weizPVP.PlayerDB[srcName] and weizPVP.PlayerDB[srcName].Estimated == true then
            --> Player exists, but data is estimated; try to get level based on ability used
            newDataFound, role = weizPVP:GetLevelData(true, event, srcName, srcGUID, arg12, arg13)
        end

        --> Check for spec and role based off spellID
        local specID = weizPVP:GetSpecFromSpell(arg12)
        if specID ~= nil then
            role = weizPVP:GetSpecRole(specID) --> "DAMAGER", "TANK", or "HEALER"
            if role then
                weizPVP:UpdatePlayerDB(srcName, nil, nil, nil, nil, true, false, role)
            end
        end

        --> Check for Stealth
        if event == "SPELL_AURA_APPLIED" and ((arg13 == "Stealth") or (arg13 == "Prowl") or (arg13 == "Invisibility")) then
            stealth = true
            dead = false
            --> Stealth Alert
            local sanctuary = UnitIsPVPSanctuary("player")
            if not sanctuary or weizPVP.Options.GLOBAL_DEBUG then --> Show alert only if not in sanctuary for Stealth players
                if arg13 ~= "Invisibility" then --> Stealth and Prowl
                    weizPVP:ShowStealthAlert("Stealth Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                else
                    weizPVP:ShowStealthAlert("Invisability Detected!", srcName, {["r"] = 0, ["b"] = 0, ["g"] = 0, ["a"] = 0.8})
                end
                weizPVP:UpdatePlayerActiveCache(srcName, GetTime(), newDataFound, nil, true, false, role)
            end
        end

        --> No new data was added or updated, send the few things we do know can update(stealth/dead)
        if newDataFound ~= true then
            weizPVP:UpdatePlayerDB(srcName, nil, nil, nil, nil, true, false)
        --> weizPVP:UpdatePlayerDB(name, class, level, race, guild, estimated, dataUpdated)
        end

        --> UPDATE
        weizPVP:UpdatePlayerActiveCache(srcName, GetTime(), newDataFound, nil, stealth, dead, role)
    end

    --> Destination Unit
    -------------------------------------------------------------------------------------------------------------------
    if (bit_band(dstFlags, OBJECT_TYPE_ENEMY) ~= 0 or weizPVP.Options.GLOBAL_DEBUG) and bit_band(dstFlags, OBJECT_TYPE_PLAYER) ~= 0 and dstGUID and dstName and dstName ~= "Unknown" then
        local newDataFound = false
        local playerData = weizPVP.PlayerDB[dstName]
        local stealth = nil
        local dead = false
        local health = nil
        if not playerData or playerData.Estimated then
            newDataFound = weizPVP:GetLevelData(false, event, dstName, dstGUID, arg12, arg13)
        end
        weizPVP:UpdatePlayerDB(dstName, nil, nil, nil, nil, nil, newDataFound)
        --> weizPVP:UpdatePlayerDB(name, class, level, race, guild, estimated, dataUpdated)

        --> if unit died
        if event == "UNIT_DIED" then
            dead = true
            health = 0
            stealth = false
        end
        weizPVP:UpdatePlayerActiveCache(dstName, GetTime(), newDataFound, health, stealth, dead, nil)
    end
end

--=> EVENT: LibCLHealth (Lib to get health from combat log)
-----------------------------------------------------------------------
if LibCLHealth then
    LibCLHealth.RegisterCallback(
        weizPVP,
        "COMBAT_LOG_HEALTH",
        function(event, unit, eventType)
            if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
                local healthPercent = tonumber(LibCLHealth.UnitHealth(unit) / UnitHealthMax(unit) * 100)
                local info = {}
                info.Name = GetUnitName(unit, true)
                if info.Name ~= "Unknown" then
                    info.Level = tonumber(UnitLevel(unit))
                    info.Dead = UnitIsDeadOrGhost(unit)
                    info.Stealth = nil
                    info.Timestamp = GetTime()
                    info.Health = healthPercent
                    weizPVP:UpdatePlayerActiveCache(info.Name, info.Timestamp, true, info.Health, info.Stealth, info.Dead, nil)
                    return true
                end
            end
        end
    )
end

--=> EVENT: Player Mouseover
-----------------------------------------------------------------------
function weizPVP:PlayerMouseoverEvent()
    weizPVP:GetUnitData("mouseover")
end

--=> EVENT: Player Target Changed
-- -----------------------------------------------------------------------
function weizPVP:PlayerTargetEvent()
    weizPVP:GetUnitData("target")
    weizPVP:ChangeTargetIcon()
end

--=> EVENT: NAMEPLATE_ADDED
-- -----------------------------------------------------------------------
function weizPVP:NameplateAdded(_, unit)
    if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
        weizPVP:GetUnitData(unit)
        local name = GetUnitName(unit, true)
        weizPVP.CurrentNameplates[name] = true
    end
end

--=> EVENT: NAMEPLATE REMOVED
-- -----------------------------------------------------------------------
function weizPVP:NameplateRemoved(_, unit)
    if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
        weizPVP:GetUnitData(unit)
        local name = GetUnitName(unit, true)
        weizPVP.CurrentNameplates[name] = nil
    end
end

--=> EVENT: ENTERING COMBAT
-----------------------------------------------------------------------
function weizPVP:EnteringCombat()
    weizPVP:SetStatusBarMessage("|cFFF4564D + IN COMBAT|r", false, 0)
end

--=> EVENT: LEAVING COMBAT
-----------------------------------------------------------------------
function weizPVP:LeavingCombat()
    weizPVP:SetStatusBarMessage("|cFF53F493 - OUT OF COMBAT|r", true, 3)
    weizPVP:ManageListTimeouts()
end

--=> EVENT: Player Levled Up
-----------------------------------------------------------------------
function weizPVP:PlayerLevelUp()
    weizPVP.Player.Level = UnitLevel("player") --> Update player level
end

--=> EVENT: Zone Changed + New Area
-----------------------------------------------------------------------
function weizPVP:ZoneChangedNewAreaEvent()
    C_Timer_After(1, GetPVPZone)
end

--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:PlayerEnteringWorldEvent()
    C_Timer_After(2, GetPVPZone)
    wipe(weizPVP.CurrentNameplates)
end

--=> EVENT: Player Entering World
-----------------------------------------------------------------------
function weizPVP:OneSecondTimer()
    weizPVP.Display.CurrentFPS = GetFramerate() -- > FPS Update

    --> Check and manage NearbyList Timeouts
    weizPVP.ManageListTimeouts()
end
