-------------------------------------------------------------------------------
--|> Lists
-- Manages the lists of players shown
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local InCombatLockdown, GetTime, RAID_CLASS_COLORS = InCombatLockdown, GetTime, RAID_CLASS_COLORS
local pairs, strlen, gsub, tinsert, sort = pairs, strlen, gsub, tinsert, sort
local C_Timer_After = C_Timer.After

-------------------------------------------------------------------------------
--|> GLOBALS
-------------------------------------------------------------------------------
weizPVP.NearbyListSize = 0
weizPVP.CurrentList = {}
weizPVP.CurrentNameplates = {}
weizPVP.NearbyList = {}
weizPVP.ActiveList = {}
weizPVP.InactiveList = {}
weizPVP.ActiveDeadList = {}
weizPVP.InactiveDeadList = {}

-------------------------------------------------------------------------------
--|> LOCALS
-------------------------------------------------------------------------------
weizPVP.playersOnBars = {}

-------------------------------------------------------------------------------
--|> FUNCTIONS
-------------------------------------------------------------------------------

--> Update Neaby Count
-------------------------------------------------------------------------------
local function UpdateNearbyCount()
    --> Upate Global
    weizPVP.NearbyCount = weizPVP.NearbyListSize
    local listSize = weizPVP.NearbyListSize

    if listSize > 0 then --> List Size is > 0
        if listSize > 50 then
            weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFFff0000" .. listSize .. "|r") --> 50+ (RED)
        elseif listSize > 40 then
            weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFFff0056" .. listSize .. "|r") --> 40+ (BRIGHT PINK)
        elseif listSize > 20 then
            weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFFff5900" .. listSize .. "|r") --> 20+ (ORANGE)
        elseif listSize > 10 then
            weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFFFFBC00" .. listSize .. "|r") --> 10+ (GOLD)
        else
            weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFFd1ff00" .. listSize .. "|r") --> 1+ (LIME GREEM)
        end
    else --> List Size is 0
        weizPVP.HeaderFrame.Text:SetText("NEARBY |cff777777:|r |cFF26FF540|r") --> 0 (GREEN)
    end
end

--> Manage List Timeouts
-------------------------------------------------------------------------------
function weizPVP:ManageListTimeouts()
    --> Checks Active and Inactive lists for inactivity and move them accordingly
    --> Refreshed the list if we found expirations and Updates the NearbyCount

    local expired = false
    local expiredCount = 0
    local i = 0
    local timestamp = GetTime()

    for player in pairs(weizPVP.ActiveList) do
        if (timestamp - weizPVP.ActiveList[player].TimeUpdated) > weizPVP.Options.Sorting.NearbyActiveTimeout and weizPVP.CurrentNameplates[player] == nil then
            weizPVP.InactiveList[player] = weizPVP.ActiveList[player]
            weizPVP.InactiveList[player].TimeAdded = timestamp + (i * 0.001)
            weizPVP.ActiveList[player] = nil
            expired = true
            i = i + 1
        end
    end
    i = 0
    for player in pairs(weizPVP.ActiveDeadList) do
        if (timestamp - weizPVP.ActiveDeadList[player].TimeUpdated) > weizPVP.Options.Sorting.NearbyActiveTimeout then
            weizPVP.InactiveDeadList[player] = weizPVP.ActiveDeadList[player]
            weizPVP.InactiveDeadList[player].TimeAdded = timestamp + (i * 0.001)
            weizPVP.ActiveDeadList[player] = nil
            expired = true
            i = i + 1
        end
    end
    for player in pairs(weizPVP.InactiveList) do
        if (timestamp - weizPVP.InactiveList[player].TimeUpdated) > weizPVP.Options.Sorting.NearbyInactiveTimeout then
            weizPVP.InactiveList[player] = nil
            weizPVP.NearbyList[player] = nil
            weizPVP.PlayerActiveCache[player] = nil
            weizPVP.CurrentList[player] = nil
            expiredCount = expiredCount + 1
            expired = true
        end
    end

    for player in pairs(weizPVP.InactiveDeadList) do
        if (timestamp - weizPVP.InactiveDeadList[player].TimeUpdated) > weizPVP.Options.Sorting.NearbyInactiveTimeout then
            weizPVP.InactiveDeadList[player] = nil
            weizPVP.NearbyList[player] = nil
            weizPVP.PlayerActiveCache[player] = nil
            weizPVP.CurrentList[player] = nil
            weizPVP.playersOnBars[player] = nil
            expiredCount = expiredCount + 1
            expired = true
        end
    end

    if expired then
        weizPVP.NearbyListSize = weizPVP.NearbyListSize - expiredCount
        UpdateNearbyCount()
        weizPVP:SortNearbyList()
        weizPVP:RefreshCurrentList()
    end
    weizPVP:SortNearbyList()
    weizPVP:RefreshCurrentList()
end

--> Clear List Data
-------------------------------------------------------------------------------
function weizPVP:ClearListData()
    wipe(weizPVP.PlayerActiveCache)
    wipe(weizPVP.ActiveList)
    wipe(weizPVP.InactiveList)
    wipe(weizPVP.NearbyList)
    wipe(weizPVP.CurrentList)
    wipe(weizPVP.playersOnBars)
    weizPVP.NearbyListSize = 0
    weizPVP:SortNearbyList()
    weizPVP:RefreshCurrentList()
    UpdateNearbyCount()
    weizPVP.TargetedIcon:SetAlpha(0)
    --> Refresh Target
    weizPVP:PlayerTargetEvent()
end

--> Format Level String
-------------------------------------------------------------------------------
local function FormatLevelString(Estimated, Level)
    local LevelText = ""
    local playerLevel = weizPVP.Player.Level

    if Level then
        --> Known Level: Get colors based on difference
        if Level == 0 then --> 0
            LevelText = "|cFFFF00CC??|r"
        elseif Level < playerLevel - 20 then --> 20+ below
            LevelText = "|cFF7cffd1" .. Level .. "|r"
        elseif Level < playerLevel - 10 then --> 10-20 below
            LevelText = "|cFF7cd1ff" .. Level .. "|r"
        elseif Level < playerLevel then --> 1-9 below
            LevelText = "|cFF7cff7f" .. Level .. "|r"
        elseif Level > playerLevel then --> Higher level
            LevelText = "|cFFf7694a" .. Level .. "|r"
        else
            LevelText = "|cFFffc863" .. Level .. "|r"
        end
    end

    if Estimated == true and Level ~= 0 then
        LevelText = LevelText .. "|cFFFF00CC+|r"
    end

    return LevelText
end

--> Update Bar
-------------------------------------------------------------------------------
--> Updates the player bar for the bar# specified
local function UpdateBar(num, name, Alpha, Health, Class, Guild, Level, Estimated, Stealth, Dead, Role)
    local Bar = weizPVP.Bars[num]
    local Button = weizPVP.Buttons[num]
    if Button ~= nil and Bar ~= nil then
        weizPVP.playersOnBars[name] = num

        if not InCombatLockdown() and name and Button then --!! need to find the root of the issue
            Button:SetAttribute("macrotext", "/target " .. name)
        end

        Button:SetAlpha(0.2)
        Bar:SetAlpha(Alpha)

        --> NAME TEXT
        local charname = gsub(name, "-(.*)", "")
        if strlen(name) > strlen(charname) then
            Bar.NameText:SetText(charname .. "|cFFFF00CC*|r")
        else
            Bar.NameText:SetText(name)
        end
        Bar.name = name

        --> Stealth
        if Stealth ~= nil and Stealth == true then
            if strlen(name) > strlen(charname) then
                Bar.NameText:SetText("|cFFffc863" .. charname .. "|r |cFFFF00CC*|r")
            else
                Bar.NameText:SetText("|cFFffc863" .. name .. "|r")
            end
        end
        --> LEVEL
        Bar.LevelText:SetText(FormatLevelString(Estimated, Level))

        --> CLASS + COLOR (BAR COLOR)
        local darkenValue = -0.05
        if Class and RAID_CLASS_COLORS[Class] then
            local classColor = RAID_CLASS_COLORS[Class]
            Bar:SetStatusBarColor(classColor.r - darkenValue, classColor.g - darkenValue, classColor.b - darkenValue, Alpha)
        end
        Bar.bg:SetVertexColor(0, 0, 0, 1)

        --> HEALTH (BAR VALUE)
        if Health ~= nil then
            Bar:SetValue(Health)
            if Health > 0 then
                Dead = false
            else
                Dead = true
            end
        end

        --> ROLE
        local adjustedRole = weizPVP:RoleAssign(Class) or Role
        if adjustedRole == "HEALER" then
            Bar.RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\healer.tga", false)
        elseif adjustedRole == "DAMAGER" then
            Bar.RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\damager.tga", false)
        elseif adjustedRole == "TANK" then
            Bar.RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\tank.tga", false)
        else
            Bar.RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\unknown.tga", false)
        end

        --> DEAD ICON
        if Dead ~= nil then
            if Dead == true then
                Bar.DEADIcon:Show()
                Bar:SetValue(0)
            elseif Dead == false then
                Bar.DEADIcon:Hide()
            end
        end

        --> GUILD TEXT
        -----------------------------------------------------------------
        local guildTxtLength = Bar.LevelText:GetWidth() + Bar.NameText:GetWidth() + 28
        if Bar.DEADIcon:IsShown() then
            guildTxtLength = guildTxtLength + 20
        end

        guildTxtLength = weizPVP.Bars[1]:GetWidth() - guildTxtLength
        Bar.GuildText:SetText(Guild)
        Bar.GuildText:SetWidth(guildTxtLength)
    end
end

--> Add Player Nearby
-------------------------------------------------------------------------------
local function UpdatePlayerLists(player, timestamp, dead, newPlayerOnList)
    -- local reSortList = false
    if dead == true or newPlayerOnList == true then
    -- reSortList = true
    end

    if weizPVP.NearbyList[player] == nil then --> ADDING NEW PLAYER
        --> New Player added; play sound?
        if weizPVP.Options.AudioAlerts.DetectedPlayerSound == true then
            weizPVP.PlaySoundAlert("Interface\\AddOns\\weizPVP\\Sounds\\detected-nearby.ogg")
        end
        ----------------------------------------------------------------------

        -- reSortList = true
        weizPVP.NearbyList[player] = {}
        weizPVP.NearbyList[player].TimeUpdated = timestamp
        weizPVP.NearbyList[player].TimeAdded = timestamp

        if dead then
            weizPVP.ActiveDeadList[player] = weizPVP.ActiveDeadList[player] or {}
            weizPVP.ActiveDeadList[player].TimeUpdated = timestamp
            weizPVP.ActiveDeadList[player].TimeAdded = timestamp
        else
            weizPVP.ActiveList[player] = weizPVP.ActiveList[player] or {}
            weizPVP.ActiveList[player].TimeAdded = timestamp
            weizPVP.ActiveList[player].TimeUpdated = timestamp
        end
    elseif not weizPVP.ActiveList[player] and not weizPVP.ActiveDeadList[player] then --> EXISTING PLAYER; WAS INACTIVE
        -- reSortList = true
        if dead then
            weizPVP.ActiveDeadList[player] = weizPVP.ActiveDeadList[player] or {}
            weizPVP.ActiveDeadList[player].TimeUpdated = timestamp
            weizPVP.ActiveDeadList[player].TimeAdded = timestamp
            weizPVP.NearbyList[player].TimeUpdated = timestamp
            weizPVP.NearbyList[player].TimeAdded = timestamp
            weizPVP.ActiveList[player] = nil
        else
            weizPVP.ActiveList[player] = weizPVP.ActiveList[player] or {}
            weizPVP.ActiveList[player].TimeUpdated = timestamp
            weizPVP.ActiveList[player].TimeAdded = timestamp
            weizPVP.NearbyList[player].TimeUpdated = timestamp
            weizPVP.NearbyList[player].TimeAdded = timestamp
            weizPVP.ActiveDeadList[player] = nil
        end

        weizPVP.InactiveList[player] = nil
        weizPVP.InactiveDeadList[player] = nil
    else --> EXISTING PLAYER; ACTIVE
        if dead == true then
            weizPVP.ActiveDeadList[player] = weizPVP.ActiveDeadList[player] or {}
            weizPVP.ActiveDeadList[player].TimeUpdated = timestamp
            weizPVP.NearbyList[player].TimeUpdated = timestamp
            weizPVP.ActiveList[player] = nil
        else
            weizPVP.ActiveList[player] = weizPVP.ActiveList[player] or {}
            weizPVP.ActiveList[player].TimeUpdated = timestamp
            weizPVP.NearbyList[player].TimeUpdated = timestamp
            weizPVP.ActiveDeadList[player] = nil
        end
        weizPVP.InactiveList[player] = nil
        weizPVP.InactiveDeadList[player] = nil
    end

    --> Sort only if we moved the player from one sublist to another
    -- if reSortList == true then
    weizPVP:SortNearbyList()
    --> Refresh all bars since we had to sort them
    weizPVP:RefreshCurrentList()
    -- else
    --     -- weizPVP:RefreshOneBar(player)
    --     weizPVP:SortNearbyList()
    --     weizPVP:RefreshCurrentList()
    -- end
end

--> Add Player Data
--> Incoming player data from event
-------------------------------------------------------------------------------
local lastTimestamp = GetTime()
function weizPVP:AddPlayerDataToNearby(player, newPlayerOnList)
    local dead = weizPVP.PlayerActiveCache[player].Dead
    local currentTime = GetTime()
    local timestamp
    if lastTimestamp >= currentTime then
        timestamp = lastTimestamp + 0.001
        lastTimestamp = timestamp
    else
        lastTimestamp = currentTime
        timestamp = lastTimestamp
    end
    if newPlayerOnList then
        weizPVP.NearbyListSize = weizPVP.NearbyListSize + 1
        UpdateNearbyCount()
    end
    UpdatePlayerLists(player, timestamp, dead, newPlayerOnList)
end

--> Manage Bars Displayed
-------------------------------------------------------------------------------
local function ManageBarsDisplayed()
    for i = 1, weizPVP.Options.Bars.MaxNumBars do
        if i > weizPVP.NearbyListSize then
            weizPVP.Bars[i]:SetAlpha(0)
            weizPVP.Bars[i]:SetValue(100)
            weizPVP.Bars[i].DEADIcon:Hide()
            weizPVP.Bars[i].RoleIcon:SetTexture("Interface\\AddOns\\weizPVP\\Icons\\unknown.tga", false)
            weizPVP.Bars[i].NameText:SetText("|cFFFFFFFF--|r")
        end
    end
end

--> Refresh Current List
-------------------------------------------------------------------------------
function weizPVP:RefreshCurrentList()
    local row = 1
    --> Refreshing all bars due to required sorting
    for k, data in pairs(weizPVP.CurrentList) do
        if row <= weizPVP.Options.Bars.MaxNumBars then
            --> Update Alpha if needed
            local Alpha = weizPVP.Options.Bars.AlphaDefault
            if weizPVP.InactiveList[data.player] or weizPVP.InactiveDeadList[data.player] then
                Alpha = weizPVP.Options.Bars.AlphaInactive
            end
            if weizPVP.ActiveDeadList[data.player] then
                Alpha = weizPVP.Options.Bars.AlphaDead
            end

            --> Update Bar
            UpdateBar(
                row,
                data.player,
                Alpha,
                weizPVP.PlayerActiveCache[data.player].Health,
                weizPVP.PlayerActiveCache[data.player].Class,
                weizPVP.PlayerActiveCache[data.player].Guild,
                weizPVP.PlayerActiveCache[data.player].Level,
                weizPVP.PlayerActiveCache[data.player].Estimated,
                weizPVP.PlayerActiveCache[data.player].Stealth,
                weizPVP.PlayerActiveCache[data.player].Dead,
                weizPVP.PlayerActiveCache[data.player].Role
            )
        else
            break
        end
        row = row + 1
    end
    weizPVP:ChangeTargetIcon()
    ManageBarsDisplayed()
end

--> Refresh One Bar
-------------------------------------------------------------------------------
function weizPVP:RefreshOneBar(player)
    --> Get Row
    local row = weizPVP.playersOnBars[player]

    --> Alpha
    local Alpha = weizPVP.Options.Bars.AlphaDefault or 0.8
    if weizPVP.InactiveList[player] or weizPVP.InactiveDeadList[player] then
        Alpha = weizPVP.Options.Bars.AlphaInactive
    end
    if weizPVP.ActiveDeadList[player] then
        Alpha = weizPVP.Options.Bars.AlphaDead
    end

    --> Update Bar
    UpdateBar(
        row,
        player,
        Alpha,
        weizPVP.PlayerActiveCache[player].Health,
        weizPVP.PlayerActiveCache[player].Class,
        weizPVP.PlayerActiveCache[player].Guild,
        weizPVP.PlayerActiveCache[player].Level,
        weizPVP.PlayerActiveCache[player].Estimated,
        weizPVP.PlayerActiveCache[player].Stealth,
        weizPVP.PlayerActiveCache[player].Dead,
        weizPVP.PlayerActiveCache[player].Role
    )

    --> Manage bars being displayed
    ManageBarsDisplayed()
end

--> Sort Nearby List
-------------------------------------------------------------------------------
function weizPVP:SortNearbyList()
    --=> Get player's into the right groups
    --> ActiveList
    local active = {}
    for player in pairs(weizPVP.ActiveList) do
        if weizPVP.NearbyList[player] then
            local position = weizPVP.NearbyList[player].TimeAdded or nil
            if position ~= nil then
                tinsert(active, {player = player, time = position})
            end
        end
    end

    --> ActiveDeadList
    local activeDead = {}
    for player in pairs(weizPVP.ActiveDeadList) do
        if weizPVP.NearbyList[player] then
            local position = weizPVP.NearbyList[player].TimeAdded or nil
            if position ~= nil then
                tinsert(activeDead, {player = player, time = position})
            end
        end
    end

    --> InactiveDeadList
    local inactiveDead = {}
    for player in pairs(weizPVP.InactiveDeadList) do
        if weizPVP.NearbyList[player] then
            local position = weizPVP.NearbyList[player].TimeAdded or nil
            if position ~= nil then
                tinsert(inactiveDead, {player = player, time = position})
            end
        end
    end

    --> InactiveList
    local inactive = {}
    for player in pairs(weizPVP.InactiveList) do
        if weizPVP.NearbyList[player] then
            local position = weizPVP.NearbyList[player].TimeAdded
            if position ~= nil then
                tinsert(inactive, {player = player, time = position})
            end
        end
    end

    sort(
        active,
        function(a, b)
            return a.time < b.time
        end
    )

    sort(
        activeDead,
        function(a, b)
            return a.time < b.time
        end
    )

    sort(
        inactive,
        function(a, b)
            return a.time < b.time
        end
    )

    sort(
        inactiveDead,
        function(a, b)
            return a.time < b.time
        end
    )

    local list = {}

    for player in pairs(active) do
        tinsert(list, active[player])
    end

    for player in pairs(activeDead) do
        tinsert(list, activeDead[player])
    end

    for player in pairs(inactive) do
        tinsert(list, inactive[player])
    end
    for player in pairs(inactiveDead) do
        tinsert(list, inactiveDead[player])
    end

    weizPVP.CurrentList = list
    weizPVP:RefreshCurrentList()
    list = {}
end

--> Play Sound Alert
-------------------------------------------------------------------------------
local playSound = true

local function EnableAlertSound()
    playSound = true
end
function weizPVP.PlaySoundAlert(url)
    if playSound == true and weizPVP.Options.AudioAlerts.DetectedPlayerSound == true then
        PlaySoundFile(url)
        playSound = false
        C_Timer_After(1, EnableAlertSound)
    end
end
